/* clwp 1.3
   header file for the co-operative lwp package,
   C.M. (Kees) Moerman, June 2002 - Januari 2003 (c)
   For documentation, see file clwp.txt
   Based on: lwp 2.0 of Josh Turpen

   Compile with -DDIAG to include error messages to standard output

   Version 1.0: changed to co-operative, added clwpDelay,
                free list, diagnostics routines
   Version 1.1: error messages enhanced, clwpLastError, clwpExit
                bug in stack handling in lwpKill (free stack???),
   Version 1.2: example4 improved
   Version 1.3 Januari 2003, C.M. Moerman (c)
                Added clwpThrow and clwpCatch
                clwpLastError is now depriciated.
                More security checks added
                All references to assert() removed
                Error messages renumbered, < 16 are fatal errors
                Bug of original LWP in clwpDeleteSemaphore solved
                clwpGetThreadPriority, clwpAdjustGetThreadPriority ifx changed
*/

#ifndef _CLWP_
#define _CLWP_

#ifndef FALSE
#define FALSE 0
#define TRUE  ((int)!FALSE)
#endif

/* User functions */

#ifdef __cplusplus
extern "C" {
#endif

extern int clwpLastError;               /* global variable containing last error number */
/* extern int _clwpLastThread;          (* internal: thread generating error */

extern int clwpOperatingMode;           /* debug/diagnostic mode */
#define CLWPMODE_STACKSIZE 1            /* enable stack depth checks */

/* ------------------------- fatal errors ---------------------- */

#define CLWPerr_cantinstallatexit 1     /* (clwpInit) Couldn't install atexit routine */
#define CLWPerr_listheadzero      2     /* (clwpKill) clwp list head is NULL! */
#define CLWPerr_stackoverflow     3     /* (clwp......) Stack damaged, probably overflow */
#define CLWPerr_null_ptr          4     /* (clwpCheck) Pointer to main task zero */
#define CLWPerr_damaged           5     /* (clwp...) Internal data structure damaged */
#define CLWPerr_circular          6     /* (clwpCheck) Circular list structure broken */
#define CLWPerr_cantkill          7     /* (clwpKill) Did not succeed in clwpKill */

/* ------------------------- usage errors ---------------------- */

#define CLWPerr_initcantmalloc   16     /* (clwpInit) Couldn't malloc clwp structure */
#define CLWPerr_pidnotfound      17     /* (clwp......) PID not found */
#define CLWPerr_stacktoosmall    18     /* (clwpSpawn) Attempt to spawn with a stack < 256 bytes */
#define CLWPerr_spawncantmalloc  19     /* (clwpSpawn) Couldn't allocate memory */
#define CLWPerr_spawncantstack   20     /* (clwpSpawn) Couldn't allocate stack */
#define CLWPerr_priozero         21     /* (clwpSpawn) Attempted to spawn with a priority of zero. */
#define CLWPerr_cantkillmain     22     /* (clwpKill) Attempted to kill main()! */
#define CLWPerr_cantkillzero     23     /* (clwpKill) Attempted to kill pid 0. */
#define CLWPerr_suspendmain      24     /* (clwpThreadSuspend) Tried to suspend MAIN */
#define CLWPerr_semacantmalloc   25     /* (clwp....Semaphore) Couldn't allocate memory */
#define CLWPerr_semaexists       26     /* (clwp....Semaphore) Semaphore already created */
#define CLWPerr_semalistzero     27     /* (clwp....Semaphore) Semaphore List is NULL */
#define CLWPerr_semanotexist     28     /* (clwp....Semaphore) Semaphore does not exist */
#define CLWPerr_semanotowner     29     /* (clwp....Semaphore) Thread deleting a semaphore it didn't create */
#define CLWPerr_semalocklimit    30     /* (clwpLockSemaphore) Lock limit exceeded */
#define CLWPerr_clwpnoinit       31     /* (clwp....) clwpInit not called */
#define CLWPerr_clwpexit           31   /*      for compatibility */
#define CLWPerr_stacksize        32     /* (clwpStackUsed) Depth analysis not enabled */
#define CLWPerr_stackmain        33     /* (clwpStackUsed) can't analyse MAIN stack */

int  clwpInit(int priority);
int  clwpSpawn(void (*proc)(void *), void *param, int stackLength, int priority, int active);
int  clwpKill(volatile int clwpid);
void clwpYield(void);
void clwpExit(void);

int  clwpCatch(void);
void clwpThrow(int errcode);

void clwpDelay(int seconds);

int clwpThreadSuspend(int clwpid);
int clwpThreadResume(int clwpid);
int clwpAdjustThreadPriority(int clwpid, int priority);
int clwpGetThreadPriority(int clwpid);

int clwpGetpid(void);
int clwpThreadCount(void);

int clwpCreateSemaphore(void *lockaddr, int count);
int clwpDeleteSemaphore(void *lockaddr);

int clwpGetSemaphoreCount(void *lockaddr);
int clwpAdjustSemaphoreCount(void *lockaddr, int count);

int clwpLockSemaphore(void *lockaddr);
int clwpReleaseSempahore(void *lockaddr);

int clwpCreateMutex(void *lockaddr);
int clwpDeleteMutex(void *lockaddr);

int clwpLockMutex(void *lockaddr);
int clwpReleaseMutex(void *lockaddr);

void clwpDump(void);
int  clwpStackUsed(int clwpid);        /* return max. bytes used */
int  clwpCheck(void);

void _clwp_gen_overflow(void * param); /* internal/test use only! */

#ifdef __cplusplus
}
#endif


#endif   /* _CLWP_ */
