/*
 * Allegro mappy framework for gaming, based on Mappy demo game
 * Stripped by C.M. Moerman, februari 2002
 * intended for tile-based games
 *
 * Version 0.1: Based on Mappy demo file
 *
 * Version 1.0 (April 19, 2002; Kees Moerman)
 * - added logging support
 * Version 1.1 (May 24, 2002; Kees Moerman)
 * - improved display selection, added options FM_8BIT, FM_ACCEL
 */
#ifndef FMLIB
#define FMLIB

#include <allegro.h>            /* probably already done in main program */

#define max(a,b) (((a)>(b))?(a):(b))
#define min(a,b) (((a)<(b))?(a):(b))

#define fm_assert(e) if(!(e)) { fm_warning(#e); fm_exit(); assert(e); }

int fm_redraw;                  /* Needed if surfaces lost (update and clear if -1) */
int fm_hwaccel;                 /* Set if hardware acceleration available */

volatile int fm_seconds;        /* total elapsed seconds */
volatile int fm_frame;          /* frame number within second */
volatile int fm_fps;            /* number of frames per second */
volatile int fm_framer;         /* software timer/counter */

BITMAP * backbuffer;            /* For hardware page flipping when fm_hwaccel */

#define FM_FULL 1               /* run full_screen */
#define FM_ISYNC 2              /* interrupt driven refresh rate */
#define FM_VSYNC 4              /* sync with vsync() */
#define FM_INFO 8               /* display frame rate etc */
#define FM_DOUBLE 16            /* update not using dirty rectangles */
#define FM_RATE 16              /* shift mask for frame rate */
#define FM_8BIT 32              /* use 8-bit graphics mode */
#define FM_ACCEL 64             /* enable hardware acceleration */
#define FM_MAPMEM 128           /* load from file in memory e.g. ALLEGRO grabber */
       
int  fm_init(int mode, int width, int height, char *mapfile);
void fm_setbackground(BITMAP *backimage);
void fm_updateparallax(int mapxoff, int mapyoff, int minx, int miny, int maxx, int maxy);
void fm_update_display(void);
void fm_exit(void);

void fm_error(char *);          /* exit due to error */
void fm_warning(char *);        /* print/wait on message */

#endif
/* end of header file */

